#include "lattsize_ocl.h"

#define  id1  0
#define  id2  Nvc
#define  id3  (Nvc*2)
#define  id4  (Nvc*3)

typedef float real_t;

//********************************************************************
__kernel void mult_xp(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 0;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ix = ist % Nx;
  int iyzt = ist/Nx;
  int nn = (ix+1) % Nx;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(nn + iyzt*Nx);
  int ig = Ndf*(ist + idir*Nst);

  for(int ic=0; ic < Ncol; ic++){
    vt1[2*ic  ] = v1[2*ic   +id1+in] - v1[2*ic+1 +id4+in];
    vt1[2*ic+1] = v1[2*ic+1 +id1+in] + v1[2*ic   +id4+in];
    vt2[2*ic  ] = v1[2*ic   +id2+in] - v1[2*ic+1 +id3+in];
    vt2[2*ic+1] = v1[2*ic+1 +id2+in] + v1[2*ic   +id3+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int ic2 = ic*Nvc;

    wt1r = u[0+ic2+ig]*vt1[0] - u[1+ic2+ig]*vt1[1]
         + u[2+ic2+ig]*vt1[2] - u[3+ic2+ig]*vt1[3]
         + u[4+ic2+ig]*vt1[4] - u[5+ic2+ig]*vt1[5];
    wt1i = u[0+ic2+ig]*vt1[1] + u[1+ic2+ig]*vt1[0]
         + u[2+ic2+ig]*vt1[3] + u[3+ic2+ig]*vt1[2]
         + u[4+ic2+ig]*vt1[5] + u[5+ic2+ig]*vt1[4];

    wt2r = u[0+ic2+ig]*vt2[0] - u[1+ic2+ig]*vt2[1]
         + u[2+ic2+ig]*vt2[2] - u[3+ic2+ig]*vt2[3]
         + u[4+ic2+ig]*vt2[4] - u[5+ic2+ig]*vt2[5];
    wt2i = u[0+ic2+ig]*vt2[1] + u[1+ic2+ig]*vt2[0]
         + u[2+ic2+ig]*vt2[3] + u[3+ic2+ig]*vt2[2]
         + u[4+ic2+ig]*vt2[5] + u[5+ic2+ig]*vt2[4];

    v2[2*ic   +id1+iv] =  wt1r;
    v2[2*ic+1 +id1+iv] =  wt1i;
    v2[2*ic   +id2+iv] =  wt2r;
    v2[2*ic+1 +id2+iv] =  wt2i;
    v2[2*ic   +id3+iv] =  wt2i;
    v2[2*ic+1 +id3+iv] = -wt2r;
    v2[2*ic   +id4+iv] =  wt1i;
    v2[2*ic+1 +id4+iv] = -wt1r;
  }

}

//********************************************************************
__kernel void mult_xm(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 0;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ix = ist % Nx;
  int iyzt = ist/Nx;
  int nn = (ix+Nx-1) % Nx;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(nn + iyzt*Nx);
  int ig = Ndf*(nn + iyzt*Nx + idir*Nst);

  for(int ic=0; ic<Ncol; ic++){
    vt1[2*ic  ] = v1[2*ic   +id1+in] + v1[2*ic+1 +id4+in];
    vt1[2*ic+1] = v1[2*ic+1 +id1+in] - v1[2*ic   +id4+in];
    vt2[2*ic  ] = v1[2*ic   +id2+in] + v1[2*ic+1 +id3+in];
    vt2[2*ic+1] = v1[2*ic+1 +id2+in] - v1[2*ic   +id3+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int icr = 2*ic;
    int ici = 2*ic+1;
    int ic1 = 0;
    int ic2 = Nvc;
    int ic3 = 2*Nvc;

    wt1r = u[icr+ic1+ig]*vt1[0] + u[ici+ic1+ig]*vt1[1]
         + u[icr+ic2+ig]*vt1[2] + u[ici+ic2+ig]*vt1[3]
         + u[icr+ic3+ig]*vt1[4] + u[ici+ic3+ig]*vt1[5];
    wt1i = u[icr+ic1+ig]*vt1[1] - u[ici+ic1+ig]*vt1[0]
         + u[icr+ic2+ig]*vt1[3] - u[ici+ic2+ig]*vt1[2]
         + u[icr+ic3+ig]*vt1[5] - u[ici+ic3+ig]*vt1[4];

    wt2r = u[icr+ic1+ig]*vt2[0] + u[ici+ic1+ig]*vt2[1]
         + u[icr+ic2+ig]*vt2[2] + u[ici+ic2+ig]*vt2[3]
         + u[icr+ic3+ig]*vt2[4] + u[ici+ic3+ig]*vt2[5];
    wt2i = u[icr+ic1+ig]*vt2[1] - u[ici+ic1+ig]*vt2[0]
         + u[icr+ic2+ig]*vt2[3] - u[ici+ic2+ig]*vt2[2]
         + u[icr+ic3+ig]*vt2[5] - u[ici+ic3+ig]*vt2[4];

    v2[icr +id1+iv] +=  wt1r;
    v2[ici +id1+iv] +=  wt1i;
    v2[icr +id2+iv] +=  wt2r;
    v2[ici +id2+iv] +=  wt2i;
    v2[icr +id3+iv] += -wt2i;
    v2[ici +id3+iv] += +wt2r;
    v2[icr +id4+iv] += -wt1i;
    v2[ici +id4+iv] += +wt1r;
  }

}

//********************************************************************
__kernel void mult_yp(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 1;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ix = ist % Nx;
  int iy = (ist/Nx) % Ny;
  int izt = ist/(Nx*Ny);
  int nn = (iy+1) % Ny;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(ix + nn*Nx + izt*Nx*Ny);
  int ig = Ndf*(ist + idir*Nst);

  for(int ic=0; ic < Ncol; ic++){
    vt1[2*ic  ] = v1[2*ic   +id1+in] + v1[2*ic   +id4+in];
    vt1[2*ic+1] = v1[2*ic+1 +id1+in] + v1[2*ic+1 +id4+in];
    vt2[2*ic  ] = v1[2*ic   +id2+in] - v1[2*ic   +id3+in];
    vt2[2*ic+1] = v1[2*ic+1 +id2+in] - v1[2*ic+1 +id3+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int ic2 = ic*Nvc;

    wt1r = u[0+ic2+ig]*vt1[0] - u[1+ic2+ig]*vt1[1]
         + u[2+ic2+ig]*vt1[2] - u[3+ic2+ig]*vt1[3]
         + u[4+ic2+ig]*vt1[4] - u[5+ic2+ig]*vt1[5];
    wt1i = u[0+ic2+ig]*vt1[1] + u[1+ic2+ig]*vt1[0]
         + u[2+ic2+ig]*vt1[3] + u[3+ic2+ig]*vt1[2]
         + u[4+ic2+ig]*vt1[5] + u[5+ic2+ig]*vt1[4];

    wt2r = u[0+ic2+ig]*vt2[0] - u[1+ic2+ig]*vt2[1]
         + u[2+ic2+ig]*vt2[2] - u[3+ic2+ig]*vt2[3]
         + u[4+ic2+ig]*vt2[4] - u[5+ic2+ig]*vt2[5];
    wt2i = u[0+ic2+ig]*vt2[1] + u[1+ic2+ig]*vt2[0]
         + u[2+ic2+ig]*vt2[3] + u[3+ic2+ig]*vt2[2]
         + u[4+ic2+ig]*vt2[5] + u[5+ic2+ig]*vt2[4];

    v2[2*ic   +id1+iv] +=  wt1r;
    v2[2*ic+1 +id1+iv] +=  wt1i;
    v2[2*ic   +id2+iv] +=  wt2r;
    v2[2*ic+1 +id2+iv] +=  wt2i;
    v2[2*ic   +id3+iv] += -wt2r;
    v2[2*ic+1 +id3+iv] += -wt2i;
    v2[2*ic   +id4+iv] +=  wt1r;
    v2[2*ic+1 +id4+iv] +=  wt1i;
  }

}

//********************************************************************
__kernel void mult_ym(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 1;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ix = ist % Nx;
  int iy = (ist/Nx) % Ny;
  int izt = ist/(Nx*Ny);
  int nn = (iy+Ny-1) % Ny;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(ix + nn*Nx + izt*Nx*Ny);
  int ig = Ndf*(ix + nn*Nx + izt*Nx*Ny + idir*Nst);

  for(int ic=0; ic<Ncol; ic++){
    vt1[2*ic  ] = v1[2*ic   +id1+in] - v1[2*ic   +id4+in];
    vt1[2*ic+1] = v1[2*ic+1 +id1+in] - v1[2*ic+1 +id4+in];
    vt2[2*ic  ] = v1[2*ic   +id2+in] + v1[2*ic   +id3+in];
    vt2[2*ic+1] = v1[2*ic+1 +id2+in] + v1[2*ic+1 +id3+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int icr = 2*ic;
    int ici = 2*ic+1;
    int ic1 = 0;
    int ic2 = Nvc;
    int ic3 = 2*Nvc;

    wt1r = u[icr+ic1+ig]*vt1[0] + u[ici+ic1+ig]*vt1[1]
         + u[icr+ic2+ig]*vt1[2] + u[ici+ic2+ig]*vt1[3]
         + u[icr+ic3+ig]*vt1[4] + u[ici+ic3+ig]*vt1[5];
    wt1i = u[icr+ic1+ig]*vt1[1] - u[ici+ic1+ig]*vt1[0]
         + u[icr+ic2+ig]*vt1[3] - u[ici+ic2+ig]*vt1[2]
         + u[icr+ic3+ig]*vt1[5] - u[ici+ic3+ig]*vt1[4];

    wt2r = u[icr+ic1+ig]*vt2[0] + u[ici+ic1+ig]*vt2[1]
         + u[icr+ic2+ig]*vt2[2] + u[ici+ic2+ig]*vt2[3]
         + u[icr+ic3+ig]*vt2[4] + u[ici+ic3+ig]*vt2[5];
    wt2i = u[icr+ic1+ig]*vt2[1] - u[ici+ic1+ig]*vt2[0]
         + u[icr+ic2+ig]*vt2[3] - u[ici+ic2+ig]*vt2[2]
         + u[icr+ic3+ig]*vt2[5] - u[ici+ic3+ig]*vt2[4];

    v2[icr +id1+iv] +=  wt1r;
    v2[ici +id1+iv] +=  wt1i;
    v2[icr +id2+iv] +=  wt2r;
    v2[ici +id2+iv] +=  wt2i;
    v2[icr +id3+iv] +=  wt2r;
    v2[ici +id3+iv] +=  wt2i;
    v2[icr +id4+iv] += -wt1r;
    v2[ici +id4+iv] += -wt1i;
  }

}

//********************************************************************
__kernel void mult_zp(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 2;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ixy = ist % (Nx*Ny);
  int iz = (ist/(Nx*Ny)) % Nz;
  int it = ist/(Nx*Ny*Nz);
  int nn = (iz+1) % Nz;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(ixy + nn*Nx*Ny + it*Nx*Ny*Nz);
  int ig = Ndf*(ist + idir*Nst);

  for(int ic=0; ic < Ncol; ic++){
    vt1[2*ic  ] = v1[2*ic   +id1+in] - v1[2*ic+1 +id3+in];
    vt1[2*ic+1] = v1[2*ic+1 +id1+in] + v1[2*ic   +id3+in];
    vt2[2*ic  ] = v1[2*ic   +id2+in] + v1[2*ic+1 +id4+in];
    vt2[2*ic+1] = v1[2*ic+1 +id2+in] - v1[2*ic   +id4+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int ic2 = ic*Nvc;

    wt1r = u[0+ic2+ig]*vt1[0] - u[1+ic2+ig]*vt1[1]
         + u[2+ic2+ig]*vt1[2] - u[3+ic2+ig]*vt1[3]
         + u[4+ic2+ig]*vt1[4] - u[5+ic2+ig]*vt1[5];
    wt1i = u[0+ic2+ig]*vt1[1] + u[1+ic2+ig]*vt1[0]
         + u[2+ic2+ig]*vt1[3] + u[3+ic2+ig]*vt1[2]
         + u[4+ic2+ig]*vt1[5] + u[5+ic2+ig]*vt1[4];

    wt2r = u[0+ic2+ig]*vt2[0] - u[1+ic2+ig]*vt2[1]
         + u[2+ic2+ig]*vt2[2] - u[3+ic2+ig]*vt2[3]
         + u[4+ic2+ig]*vt2[4] - u[5+ic2+ig]*vt2[5];
    wt2i = u[0+ic2+ig]*vt2[1] + u[1+ic2+ig]*vt2[0]
         + u[2+ic2+ig]*vt2[3] + u[3+ic2+ig]*vt2[2]
         + u[4+ic2+ig]*vt2[5] + u[5+ic2+ig]*vt2[4];

    v2[2*ic   +id1+iv] +=  wt1r;
    v2[2*ic+1 +id1+iv] +=  wt1i;
    v2[2*ic   +id2+iv] +=  wt2r;
    v2[2*ic+1 +id2+iv] +=  wt2i;
    v2[2*ic   +id3+iv] +=  wt1i;
    v2[2*ic+1 +id3+iv] += -wt1r;
    v2[2*ic   +id4+iv] += -wt2i;
    v2[2*ic+1 +id4+iv] +=  wt2r;
  }

}

//********************************************************************
__kernel void mult_zm(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 2;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ixy = ist % (Nx*Ny);
  int iz = (ist/(Nx*Ny)) % Nz;
  int it = ist/(Nx*Ny*Nz);
  int nn = (iz+Nz-1) % Nz;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(ixy + nn*Nx*Ny + it*Nx*Ny*Nz);
  int ig = Ndf*(ixy + nn*Nx*Ny + it*Nx*Ny*Nz + idir*Nst);

  for(int ic=0; ic < Ncol; ic++){
    vt1[2*ic  ] = v1[2*ic   +id1+in] + v1[2*ic+1 +id3+in];
    vt1[2*ic+1] = v1[2*ic+1 +id1+in] - v1[2*ic   +id3+in];
    vt2[2*ic  ] = v1[2*ic   +id2+in] - v1[2*ic+1 +id4+in];
    vt2[2*ic+1] = v1[2*ic+1 +id2+in] + v1[2*ic   +id4+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int icr = 2*ic;
    int ici = 2*ic+1;
    int ic1 = 0;
    int ic2 = Nvc;
    int ic3 = 2*Nvc;

    wt1r = u[icr+ic1+ig]*vt1[0] + u[ici+ic1+ig]*vt1[1]
         + u[icr+ic2+ig]*vt1[2] + u[ici+ic2+ig]*vt1[3]
         + u[icr+ic3+ig]*vt1[4] + u[ici+ic3+ig]*vt1[5];
    wt1i = u[icr+ic1+ig]*vt1[1] - u[ici+ic1+ig]*vt1[0]
         + u[icr+ic2+ig]*vt1[3] - u[ici+ic2+ig]*vt1[2]
         + u[icr+ic3+ig]*vt1[5] - u[ici+ic3+ig]*vt1[4];

    wt2r = u[icr+ic1+ig]*vt2[0] + u[ici+ic1+ig]*vt2[1]
         + u[icr+ic2+ig]*vt2[2] + u[ici+ic2+ig]*vt2[3]
         + u[icr+ic3+ig]*vt2[4] + u[ici+ic3+ig]*vt2[5];
    wt2i = u[icr+ic1+ig]*vt2[1] - u[ici+ic1+ig]*vt2[0]
         + u[icr+ic2+ig]*vt2[3] - u[ici+ic2+ig]*vt2[2]
         + u[icr+ic3+ig]*vt2[5] - u[ici+ic3+ig]*vt2[4];

    v2[icr +id1+iv] +=  wt1r;
    v2[ici +id1+iv] +=  wt1i;
    v2[icr +id2+iv] +=  wt2r;
    v2[ici +id2+iv] +=  wt2i;
    v2[icr +id3+iv] += -wt1i;
    v2[ici +id3+iv] +=  wt1r;
    v2[icr +id4+iv] +=  wt2i;
    v2[ici +id4+iv] += -wt2r;
  }

}
//********************************************************************
__kernel void mult_tp(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 3;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ixyz = ist % (Nx*Ny*Nz);
  int it = ist/(Nx*Ny*Nz);
  int nn = (it+1) % Nt;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(ixyz + nn*Nx*Ny*Nz);
  int ig = Ndf*(ist + idir*Nst);

  for(int ic=0; ic < Ncol; ic++){
    vt1[2*ic  ] = 2.0 * v1[2*ic   +id3+in];
    vt1[2*ic+1] = 2.0 * v1[2*ic+1 +id3+in];
    vt2[2*ic  ] = 2.0 * v1[2*ic   +id4+in];
    vt2[2*ic+1] = 2.0 * v1[2*ic+1 +id4+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int ic2 = ic*Nvc;

    wt1r = u[0+ic2+ig]*vt1[0] - u[1+ic2+ig]*vt1[1]
         + u[2+ic2+ig]*vt1[2] - u[3+ic2+ig]*vt1[3]
         + u[4+ic2+ig]*vt1[4] - u[5+ic2+ig]*vt1[5];
    wt1i = u[0+ic2+ig]*vt1[1] + u[1+ic2+ig]*vt1[0]
         + u[2+ic2+ig]*vt1[3] + u[3+ic2+ig]*vt1[2]
         + u[4+ic2+ig]*vt1[5] + u[5+ic2+ig]*vt1[4];

    wt2r = u[0+ic2+ig]*vt2[0] - u[1+ic2+ig]*vt2[1]
         + u[2+ic2+ig]*vt2[2] - u[3+ic2+ig]*vt2[3]
         + u[4+ic2+ig]*vt2[4] - u[5+ic2+ig]*vt2[5];
    wt2i = u[0+ic2+ig]*vt2[1] + u[1+ic2+ig]*vt2[0]
         + u[2+ic2+ig]*vt2[3] + u[3+ic2+ig]*vt2[2]
         + u[4+ic2+ig]*vt2[5] + u[5+ic2+ig]*vt2[4];

    v2[2*ic   +id3+iv] +=  wt1r;
    v2[2*ic+1 +id3+iv] +=  wt1i;
    v2[2*ic   +id4+iv] +=  wt2r;
    v2[2*ic+1 +id4+iv] +=  wt2i;
  }

}
//********************************************************************
__kernel void mult_tm(__global real_t *v2, __global real_t *u,
	      		       	     	      __global real_t *v1){

  int idir = 3;

  real_t vt1[Nvc], vt2[Nvc];
  real_t wt1r, wt1i, wt2r, wt2i;

  int ist = get_global_id(0);

  int ixyz = ist % (Nx*Ny*Nz);
  int it = ist/(Nx*Ny*Nz);
  int nn = (it+Nt-1) % Nt;

  int iv = Nvc*ND*ist;
  int in = Nvc*ND*(ixyz + nn*Nx*Ny*Nz);
  int ig = Ndf*(ixyz + nn*Nx*Ny*Nz + idir*Nst);

  for(int ic=0; ic < Ncol; ic++){
    vt1[2*ic  ] = 2.0 * v1[2*ic   +id1+in];
    vt1[2*ic+1] = 2.0 * v1[2*ic+1 +id1+in];
    vt2[2*ic  ] = 2.0 * v1[2*ic   +id2+in];
    vt2[2*ic+1] = 2.0 * v1[2*ic+1 +id2+in];
  }

  for(int ic=0; ic < Ncol; ic++){
    int icr = 2*ic;
    int ici = 2*ic+1;
    int ic1 = 0;
    int ic2 = Nvc;
    int ic3 = 2*Nvc;
    wt1r = u[icr+ic1+ig]*vt1[0] + u[ici+ic1+ig]*vt1[1]
         + u[icr+ic2+ig]*vt1[2] + u[ici+ic2+ig]*vt1[3]
         + u[icr+ic3+ig]*vt1[4] + u[ici+ic3+ig]*vt1[5];
    wt1i = u[icr+ic1+ig]*vt1[1] - u[ici+ic1+ig]*vt1[0]
         + u[icr+ic2+ig]*vt1[3] - u[ici+ic2+ig]*vt1[2]
         + u[icr+ic3+ig]*vt1[5] - u[ici+ic3+ig]*vt1[4];
    wt2r = u[icr+ic1+ig]*vt2[0] + u[ici+ic1+ig]*vt2[1]
         + u[icr+ic2+ig]*vt2[2] + u[ici+ic2+ig]*vt2[3]
         + u[icr+ic3+ig]*vt2[4] + u[ici+ic3+ig]*vt2[5];
    wt2i = u[icr+ic1+ig]*vt2[1] - u[ici+ic1+ig]*vt2[0]
         + u[icr+ic2+ig]*vt2[3] - u[ici+ic2+ig]*vt2[2]
         + u[icr+ic3+ig]*vt2[5] - u[ici+ic3+ig]*vt2[4];
    v2[icr +id1+iv] +=  wt1r;
    v2[ici +id1+iv] +=  wt1i;
    v2[icr +id2+iv] +=  wt2r;
    v2[ici +id2+iv] +=  wt2i;
  }

}

//********************************************************************
__kernel void spxgm5(__global real_t *v, __global real_t *w, real_t CKs){

  int ist = get_global_id(0);
  int iv  = Nvc*ND * ist;

  for(int ivc=0; ivc < Nvc; ivc++){
    real_t v3 = v[ivc+id3+iv];
    real_t v4 = v[ivc+id4+iv];
    v[ivc+id3+iv] = w[ivc+id1+iv] - CKs * v[ivc+id1+iv];
    v[ivc+id4+iv] = w[ivc+id2+iv] - CKs * v[ivc+id2+iv];
    v[ivc+id1+iv] = w[ivc+id3+iv] - CKs * v3;
    v[ivc+id2+iv] = w[ivc+id4+iv] - CKs * v4;
  }

}

//********************************************************************
__kernel void mult_all(__global real_t *v2, __global real_t *u,
	       	     	      __global real_t *v1, real_t CKs){

  mult_xp(v2,u,v1);
  mult_xm(v2,u,v1);
  mult_yp(v2,u,v1);
  mult_ym(v2,u,v1);
  mult_zp(v2,u,v1);
  mult_zm(v2,u,v1);
  mult_tp(v2,u,v1);
  mult_tm(v2,u,v1);

  spxgm5(v2, v1, CKs);

}
//************************************************************END*****
